<?php
/**
 * FlickrImport
 *
 * Import your flickr stream into your pixelpost stream.
 *
 * IMPORTANT :
 *
 * This Pixelpost Addon include a modified part of Phlickr
 * produced by Andrew Morton <drewish@katherinehouse.com>
 * (@link http://sourceforge.net/projects/phlickr/)
 * which is under LGPL license.
 *
 * This Pixelpost addon is heavily inspired by Pixelpost Importr addon produced
 * by Christian Cueni <chrigu@trivialview.ch> (@link http://www.trivialview.ch)
 * which is under GPL license.
 *
 * REQUIREMENTS :
 *
 * - PHP CURL library installed
 * - SimpleXml library installed
 *
 * LICENCE :
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * @version 0.03
 * @author Alban Leroux <seza@paradoxal.org>
 * @see http://www.paradoxal.org/blog/
 * @license GPL License: http://www.gnu.org/copyleft/gpl.html
 */

// set up the database prefix constant
defined('FlickrImportDbPrefix') or
define('FlickrImportDbPrefix', $pixelpost_db_prefix, true);

/**
 * The common methods
 */
class FlickrImport
{
    const KEY_API = '9a4cf51154935a525c7e1e0f692017e1';
    const SECRET  = '21675f692ee9720c';
    const VERSION = '0.03';
    const NAME    = 'FlickrImport';

    /**
     * Return a new PhlickApi
     *
     * @param  $token Add it if the user authorize the application;
     * @return Phlickr_Api
     */
    public static function getApi($token = null)
    {
        return new Phlickr_Api(self::KEY_API, self::SECRET, $token);
    }

    /**
     * Is script launched from command line
     *
     * @return bool
     */
    public static function isCmdLine()
    {
        return ! isset($_SERVER['REQUEST_URI']);
    }

    /**
     * Return the correct endline ( \n or <br /> )
     *
     * @return string
     */
    public static function getEndLine()
    {
        return (self::isCmdLine() ?  "\n" : '<br />');
    }

    /**
     * Add the Phlickr library to the include_path
     */
    public static function registerPhlickrLibrary()
    {
        // where is Phlickr library
        $phlickrPath = dirname(__FILE__);

        // add it to include path
        ini_set('include_path', ini_get('include_path') . PATH_SEPARATOR . $phlickrPath);

        // include base class
        require_once 'Phlickr/Api.php';
        require_once 'Phlickr/User.php';
        require_once 'Phlickr/CommentList.php';
        require_once 'Phlickr/Comment.php';
    }


    /**
     * Print mysql's errors
     *
     * @return bool (always false)
     */
    public static function mysqlError()
    {
        // print the mysql error
       echo mysql_error();
       return false;
    }

    /**
     * Set up the database connection
     *
     * @global string $pixelpost_db_host
     * @global string $pixelpost_db_user
     * @global string $pixelpost_db_pass
     * @global string $pixelpost_db_pixelpost
     */
    public static function mysqlConnect()
    {
        // global database informations (erk !!)
        global $pixelpost_db_host;
        global $pixelpost_db_user;
        global $pixelpost_db_pass;
        global $pixelpost_db_pixelpost;

        // make connection
        mysql_connect($pixelpost_db_host, $pixelpost_db_user, $pixelpost_db_pass)
            or exit(self::mysqlError());
        
        // select db
        mysql_select_db($pixelpost_db_pixelpost) or exit(self::mysqlError());
    }

    /**
     * Run a mysql query and return the result or false if there is an error
     *
     * @param  string    $query the query
     * @param  array     $datas the paramters to the query (like sprintf)
     * @return ressource
     */
    public static function mysqlQuery($query, array $datas = array())
    {
        // add the db prefix to the vsprintf data (in first)
        array_unshift($datas, FlickrImportDbPrefix);

        // run the mysql query
        $query = mysql_query(vsprintf($query, $datas));

        // check if well done or print the error
        ($query !== false) or self::mysqlError();

        // return the result
        return $query;
    }

    /**
     * Exec a mysql query and return true or false.
     *
     * @param  string $query the query
     * @param  array  $datas the paramters to the query (like sprintf)
     * @return bool
     */
    public static function mysqlExec($query, array $datas = array())
    {
        $result = self::mysqlQuery($query, $datas);

        return ($result !== false);
    }

    /**
     * Exec a mysql query and test if there is a result
     *
     * @param  string $query the query
     * @param  array  $datas the paramters to the query (like sprintf)
     * @return bool
     */
    public static function mysqlIsResult($query, array $datas = array())
    {
        $result = self::mysqlQuery($query, $datas);

        if ($result === false) return false;

        return (mysql_result($result, 0) !== false);
    }

    /**
     * Exec a mysql count query and return the count result
     * 
     * @param  string $query the query
     * @param  array  $datas the paramters to the query (like sprintf)
     * @return int
     */
    public static function mysqlCount($query, array $datas = array())
    {
        $result = self::mysqlQuery($query, $datas);

        return (($result === false) ? 0 : intval(mysql_result($result, 0)));
    }

    /**
     * Install plugins tables in database
     */
    public static function install()
    {
        // Link Pixelpost's photo and Flickr's photo
        self::mysqlExec('CREATE TABLE IF NOT EXISTS `%sFlickrImport_assoc` (
            `flickr_id` varchar(20) COLLATE utf8_unicode_ci NOT NULL,
            `pp_id` int(11) NOT NULL,
            PRIMARY KEY (`flickr_id`));');

        // Link Pixelpost's category and Flickr's photoset
        self::mysqlExec('CREATE TABLE IF NOT EXISTS `%sFlickrImport_photosets` (
            `id` INT NOT NULL AUTO_INCREMENT,
            `flickr_id` VARCHAR(255) NOT NULL,
            `cat_id` INT(11) NULL DEFAULT NULL,
            PRIMARY KEY (`id`));');

        // Link Pixelpost's category and Flickr's photoset
        self::mysqlExec('CREATE TABLE IF NOT EXISTS `%sFlickrImport_tags` (
            `id` INT NOT NULL AUTO_INCREMENT,
            `tag` VARCHAR(255) NOT NULL,
            PRIMARY KEY (`id`));');

        // Configuration table
        self::mysqlExec('CREATE TABLE IF NOT EXISTS `%sFlickrImport_conf` (
            `id` INT NOT NULL AUTO_INCREMENT,
            `name` VARCHAR(50) NOT NULL,
            `value` VARCHAR(255) NOT NULL,
            PRIMARY KEY (`id`));');

        // populate conf table
        self::mysqlExec('INSERT INTO `%sFlickrImport_conf` (`name`, `value`) VALUES
            ("version", "' . self::VERSION . '"),
            ("token", ""),
            ("photo.size", "original"),
            ("photo.date", "taken"),
            ("photo.resize", "no"),
            ("photo.resize.size", "830"),
            ("photo.resize.quality", "85"),
            ("photo.link", "no"),
            ("photo.link.text", "see it on Flickr"),
            ("photo.link.format", "[ %s ]"),
            ("photo.tags", "yes"),
            ("thumbs.size", "square"),
            ("comments", "yes"),
            ("comments.publish", "yes"),
            ("comments.html", "no");');
    }

    public static function makeUpdate()
    {
        // actual version is 0.01 or 0.02
        if (self::mysqlIsResult('SHOW TABLES LIKE "%sFlickrImport_config";'))
        {
            // retreive the old conf option
            $query = self::mysqlQuery('SELECT `token`, `comments`, `size`, `html`
                FROM `%sFlickrImport_config` LIMIT 1;');

            $conf = mysql_fetch_assoc($query);

            // translate old names in new names
            $conf['thumbs.size']      = $conf['size'];
            $conf['comments.html']    = $conf['html'];
            $conf['comments.publish'] = 'no';

            if ($conf['comments'] == 'yes-publish')
            {
                $conf['comments'] == 'yes';
                $conf['comments.publish'] = 'yes';
            }

            unset($conf['html']);
            unset($conf['size']);

            // add them to the new table configuration
            self::setOptions($conf);

            // drop old table config
            self::mysqlExec('DROP TABLE `%sFlickrImport_config`;');
        }
/*
NO ACTUAL NEED, BUT FOR FUTURE CERTAINLY...

        // get the database version (transform 0.03 to int like 3)
        $dataVersion = intval(self::mysqlCount('SELECT `value` * 100 FROM
            `%sFlickrImport_conf` WHERE `name` = "version" LIMIT 1;'));

        // get the code version (transform 0.03 to int like 3)
        $codeVersion = intval(round(self::VERSION * 100, 0));

        // check if database need update
        if ($dataVersion < $codeVersion)
        {
            // this is an example
            if ($dataVersion < 3)
            {
                // database update code here...
            }
        }
*/
    }

    /**
     * Check if table's plugins is created in database
     *
     * @return bool
     */
    public static function isInstalled()
    {
        return self::mysqlIsResult('SHOW TABLES LIKE "%sFlickrImport_conf";');
    }

    /**
     * Change FlickrImport user's options
     *
     * @return array $datas 
     * @return bool
     */
    public static function setOptions(array $datas)
    {
        // get the actual conf
        $conf = self::getOptions();

        // remove datas which no change
        foreach($datas as $name => $value)
        {
            // if value not change
            if (isset($conf[$name]) && $conf[$name] == $value) continue;

            if (isset($conf[$name]))
            {
                // if value exists update it
                self::mysqlExec('UPDATE `%sFlickrImport_conf` SET `value` = "%s"
                    WHERE `name` = "%s" LIMIT 1;', array(mysql_real_escape_string($value),
                                                         mysql_real_escape_string($name)));
            }
            else
            {
                // else insert it
                self::mysqlExec('INSERT INTO `%sFlickrImport_conf` (`name`, `value`)
                    VALUES ("%s", "%s");', array(mysql_real_escape_string($name),
                                                 mysql_real_escape_string($value)));
            }
        }

        return true;
    }

    /**
     * Retreive FlickrImport user's options
     *
     * @return array
     */
    public static function getOptions()
    {
        $query = self::mysqlQuery('SELECT `name`, `value` FROM `%sFlickrImport_conf`;');

        $conf = array();

        while (false !== $row = mysql_fetch_assoc($query))
        {
            $conf[$row['name']] = $row['value'];
        }

        return $conf;
    }

    /**
     * Forget stored Photo's flickr Id
     *
     * @param  int  $photoId Pixelpost's photo ID
     * @return bool
     */
    public static function delPhoto($photoId)
    {
        return self::mysqlExec('DELETE FROM `%sFlickrImport_assoc`
            WHERE `pp_id` = %d;', array(intval($photoId)));
    }

    /**
     * Register a Photo's flickr Id
     *
     * @param  string $flickrId Flickr's photo Id
     * @param  array  $datas    Data's photo with an array indexed like that :
     *                          flickrId, date, head, body, image, altHead,
     *                          altBody, comments, exif
     * @return bool
     */
    public static function addPhoto($flickrId, array $datas)
    {
        self::mysqlExec('INSERT INTO `%spixelpost` (`datetime`, `headline`,
            `body`, `image`, `alt_headline`, `alt_body`, `comments`, `exif_info`)
            VALUES ("%s", "%s", "%s", "%s", "%s", "%s", "%s", "%s");', array(
                          mysql_real_escape_string($datas['date']),
                          mysql_real_escape_string($datas['head']),
                          mysql_real_escape_string($datas['body']),
                          mysql_real_escape_string($datas['image']),
                          mysql_real_escape_string($datas['altHead']),
                          mysql_real_escape_string($datas['altBody']),
                          mysql_real_escape_string($datas['comments']),
                          $datas['exifs']));

        $photoId = intval(mysql_insert_id());

        return self::mysqlExec('INSERT INTO `%sFlickrImport_assoc` (`flickr_id`,
            `pp_id`) VALUES ("%s", %d);', array($flickrId, $photoId));
    }

    /**
     * Get pixelpost ID of Photo's flickr Id
     *
     * @param  string $flickrId Flickr's photo ID
     * @return int
     */
    public static function getPhoto($flickrId)
    {
        return self::mysqlCount('SELECT `pp_id` FROM `%sFlickrImport_assoc`
            WHERE `flickr_id` = "%s" LIMIT 1;', array($flickrId));
    }

    /**
     * Is Registred Photo's flickr Id
     *
     * @param  string $flickrId Flickr's photo ID
     * @return bool
     */
    public static function isPhoto($flickrId)
    {
        return self::mysqlCount('SELECT COUNT(`flickr_id`) FROM `%sFlickrImport_assoc`
            WHERE `flickr_id` = "%s";', array($flickrId));
    }

    /**
     * Change the list of Flickr tag to avoid when sync
     *
     * @param  array $photosets Array of Flickr's photoset Id to Sync
     * @return bool
     */
    public static function updateNotSyncTags(array $tags)
    {
        // delete all photosets
        $result = self::mysqlExec('DELETE FROM `%sFlickrImport_tags`;');

        // insert new set of photoset
        foreach($tags as $tag)
        {
            self::mysqlExec('INSERT INTO `%sFlickrImport_tags` (`tag`)
                VALUES ("%s");', array(mysql_real_escape_string($tag)));
        }

        return true;
    }

    /**
     * Return all Flickr's tags to avoid when sync
     *
     * @return array
     */
    public static function getNotSyncTags()
    {
        // get all photosets
        $result = self::mysqlQuery('SELECT `tag` FROM `%sFlickrImport_tags`;');

        $list = array();

        while(false !== $row = mysql_fetch_row($result)) $list[$row[0]] = $row[0];

        return $list;
    }

    /**
     * Change the list of Flickr photoset to sync
     *
     * @param  array $photosets Array of Flickr's photoset Id to Sync
     * @return bool
     */
    public static function updateSyncPhotoset(array $photosets)
    {
        // backup prev photosets
        $result = self::mysqlQuery('SELECT `flickr_id`, `cat_id` FROM `%sFlickrImport_photosets`;');

        $list = array();

        while(false !== $row = mysql_fetch_row($result)) 
        {
            $list[$row[0]] = $row[1];
        }

        // delete all photosets
        $result = self::mysqlExec('DELETE FROM `%sFlickrImport_photosets`;');

        // insert new set of photoset
        foreach($photosets as $id)
        {
            self::mysqlExec('INSERT INTO `%sFlickrImport_photosets` (`flickr_id`)
                VALUES ("%s");', array(mysql_real_escape_string($id)));
        }

        // restore linking with category
        // for all delete photosets, link are lost
        foreach($list as $flickrId => $catId)
        {
            self::mysqlExec('UPDATE `%sFlickrImport_photosets` SET `cat_id` = %d WHERE
                `flickr_id` = "%s";', array($catId, mysql_real_escape_string($flickrId)));
        }

        return true;
    }

    /**
     * Return all Flickr's photosetId to sync
     *
     * @return array
     */
    public static function getSyncPhotoset()
    {
        // clean the break link between cat_id and flickr_id before return the
        // result
        $result = self::mysqlQuery('SELECT p.`id` FROM `%sFlickrImport_photosets` p
            LEFT OUTER JOIN `%scategories` c ON c.`id` = p.`cat_id`
            WHERE p.`cat_id` IS NOT NULL AND c.`id` IS NULL;', array(FlickrImportDbPrefix));

        $list = array();

        while(false !== $row = mysql_fetch_row($result)) $list[] = $row[0];

        foreach($list as $id) 
        {
            self::mysqlExec('UPDATE `%sFlickrImport_photosets` SET `cat_id` = NULL
                WHERE `id` = %d;', array(intval($id)));
        }

        // get all photosets
        $result = self::mysqlQuery('SELECT `flickr_id` FROM `%sFlickrImport_photosets`;');

        $list = array();

        while(false !== $row = mysql_fetch_row($result)) $list[$row[0]] = $row[0];

        return $list;
    }

    /**
     * Add a photoset
     *
     * @param  string $flickrId Flickr photoset Id
     * @param  string $name     Pixelpost photoset Name
     * @param  string $alt      Pixelpost photoset Description
     * @return bool
     */
    public static function addPhotoset($flickrId, $name, $alt)
    {
        // check if a same named cat exists
        $count = self::mysqlCount('SELECT `id` FROM `%scategories`
            WHERE `name` = "%s" LIMIT 1;', array(mysql_real_escape_string($name)));

        // if no create it
        if ($count == 0)
        {
            self::mysqlExec('INSERT INTO `%scategories` (`name`, `alt_name`)
                VALUES ("%s", "%s");', array(mysql_real_escape_string($name),
                                             mysql_real_escape_string($alt)));

            $catId = intval(mysql_insert_id());
        }
        else
        {
            $catId = intval($count);
        }

        // link photoset and category
        return self::mysqlExec('UPDATE `%sFlickrImport_photosets` SET `cat_id` = %d WHERE
            `flickr_id` = "%s";', array($catId, mysql_real_escape_string($flickrId)));
    }

    /**
     * Get a photoset ID
     *
     * @param  string $flickrId Flickr photoset Id
     * @return int
     */
    public static function getPhotoset($flickrId)
    {
        return self::mysqlCount('SELECT `cat_id` FROM `%sFlickrImport_photosets` WHERE
            `flickr_id` = "%s" LIMIT 1;', array(mysql_real_escape_string($flickrId)));
    }

    /**
     * Is a photoset allready registred
     * 
     * @param  string $flickrId Flickr photoset Id
     * @return bool
     */
    public static function isPhotoset($flickrId)
    {
        return (bool) self::mysqlCount('SELECT COUNT(`id`) FROM `%sFlickrImport_photosets`
            WHERE `flickr_id` = "%s" AND `cat_id` IS NOT NULL LIMIT 1;',
                array(mysql_real_escape_string($flickrId)));
    }

    /**
     * Add photo in photoset
     *
     * @param  int  $photoId    Pixelpost photo ID
     * @param  int  $photosetId Pixelpost photoset ID
     * @return bool
     */
    public static function addInPhotoset($photoId, $photosetId)
    {
        return self::mysqlExec('INSERT INTO `%scatassoc` (`image_id`, `cat_id`)
            VALUES (%d, %d);', array(intval($photoId), intval($photosetId)));
    }

    /**
     * Is photo is in photoset
     * 
     * @param  int  $photoId    Pixelpost photo ID
     * @param  int  $photosetId Pixelpost photoset ID
     * @return bool
     */
    public static function isInPhotoset($photoId, $photosetId)
    {
        return (bool) self::mysqlCount('SELECT COUNT(`id`) FROM `%scatassoc`
            WHERE `image_id` = %d AND `cat_id` = %d LIMIT 1;', 
                array(intval($photoId), intval($photosetId)));
    }

    /**
     * Check if a flickr's photo comment allready exist
     *
     * @param  int  $commentId Flickr's comment Id
     * @param  int  $photoId   Pixelpost photo Id
     * @return bool
     */
    public static function isComment($commentId, $photoId)
    {
        return (bool) self::mysqlCount('SELECT COUNT(`id`) FROM `%scomments`
            WHERE `parent_id` = %d AND `signature` = "%s" LIMIT 1;', 
                array(intval($photoId), $commentId));
    }

    /**
     * Add a comment
     *
     * @param  array $datas   All datas of the comment
     * @param  bool  $publish TRUE publish the comment, FALSE keep in moderation
     * @return bool
     */
    public static function addComment(array $datas, $publish = false)
    {
        return self::mysqlQuery('INSERT INTO `%scomments` (`parent_id`, `datetime`,
            `ip`,  `message`,  `name`,  `url`, `publish`, `signature`) VALUES
            (%d, "%s", "%s", "%s", "%s", "%s", "%s", "%s");',
                array(intval($datas['photo']),
                     mysql_real_escape_string($datas['date']),
                     '127.0.0.1',
                     mysql_real_escape_string($datas['message']),
                     mysql_real_escape_string($datas['name']),
                     mysql_real_escape_string($datas['url']),
                     ($publish ? 'yes' : 'no'),
                     mysql_real_escape_string($datas['id'])));
    }

    /**
     * Resize an image and store it in jpeg (if resize will be done). Return the
     * new filename if resize will be done.
     *
     * @param  string $path    The actual image path (without trailling slash)
     * @param  string $file    The actual image filename
     * @param  int    $size    The max longer side size
     * @param  int    $quality The quality of compression of resized image
     * @return string
     */
    public static function resize($path, $file, $size, $quality)
    {
        // the image
        $img = null;
        // the image extension
        $ext = strtolower(end(explode('.', $file)));

        // try to load the image with GD
        switch($ext)
        {
            case 'jpg'  : $img = @imagecreatefromjpeg($path . DIRECTORY_SEPARATOR . $file); break;
            case 'jpeg' : $img = @imagecreatefromjpeg($path . DIRECTORY_SEPARATOR . $file); break;
            case 'png'  : $img = @imagecreatefrompng($path . DIRECTORY_SEPARATOR . $file);  break;
            case 'gif'  : $img = @imagecreatefromgif($path . DIRECTORY_SEPARATOR . $file);  break;
            default     : return $file;
        }

        // if no result then exit
        if( ! $img) return $file;

        // get and calc size information
        $width    = imagesx($img);
        $height   = imagesy($img);
        $longSide = max($width, $height);
        $scale    = $size / $longSide;

        // if longer side is smaller than the resize size, do nothing
        if ($longSide <= $size) 
        {
            imagedestroy($img);
            return $file;
        }

        // prepare and create the new image
        $newWidth  = intval(round($scale * $width, 0));
        $newHeight = intval(round($scale * $height, 0));
        $newImg    = imagecreatetruecolor($newWidth, $newHeight);
        $newFile   = substr($file, 0, - strlen($ext)) . 'jpg';

        // gd 2.0.1 or later: imagecopyresampled, less than 2.0: imagecopyresized
        if (function_exists(imagecopyresampled))
            imagecopyresampled($newImg, $img, 0, 0, 0, 0, $newWidth, $newHeight, $width, $height);
        else
            imagecopyresized($newImg, $img, 0, 0, 0, 0, $newWidth, $newHeight, $width, $height);

        // delete the original image
        imagedestroy($img);
        unlink($file);

        // register the new image
        touch($newFile);
        chmod($newFile, 0644);
        imagejpeg($newImg, $path . DIRECTORY_SEPARATOR . $newFile, $quality);

        // return the new file name
        return $newFile;
    }
}